use std::{
    fs::File,
    io::BufReader,
    thread,
};
use std::collections::HashMap;
use std::io::{BufRead, Seek, SeekFrom};
use std::sync::mpsc;
use std::time::Instant;

use onebrc::{parse_line, parse_temp};

const DEFAULT_HASHMAP_LENGTH: usize = 10000;

fn main() {
    const FILE_PATH: &str = "../../../measurements.txt";
    let now = Instant::now();
    thread::scope(|s| {
        let mut stations: HashMap<String, onebrc::StationMeasurements> = HashMap::with_capacity(DEFAULT_HASHMAP_LENGTH);
        let (tx, rx) = mpsc::channel();
        let cores = thread::available_parallelism().unwrap().into();
        let file = File::open(FILE_PATH).expect("File measurements.txt not found");
        let mut reader = BufReader::new(&file);
        let file_length = reader.seek(SeekFrom::End(0)).unwrap();
        let chunk_length = file_length as usize / cores;
        let mut bounds = Vec::with_capacity(cores + 1);
        bounds.push(0);
        for i in 1..cores {
            let mut reader = BufReader::new(&file);
            let mut byte_start = chunk_length * i;
            reader.seek(SeekFrom::Start(byte_start as u64)).expect("could not seek");
            let mut line = Vec::with_capacity(108);
            let line_len = reader.read_until(b'\n', &mut line).expect("could not read bytes");
            byte_start += line_len;
            bounds.push(byte_start as u64);
        }
        bounds.push(file_length);
        for i in 0..cores {
            let tx = tx.clone();
            let mut currposition = *bounds.get(i).unwrap();
            let end =  *bounds.get(i+1).unwrap();
            s.spawn(move || {
                let file = File::open(FILE_PATH).expect("File measurements.txt not found");
                let mut reader = BufReader::new(&file);
                reader.seek(SeekFrom::Start(currposition)).unwrap();
                let mut t_stations: HashMap<String, onebrc::StationMeasurements> =
                    HashMap::with_capacity(DEFAULT_HASHMAP_LENGTH);
                let mut line = Vec::with_capacity(108);
                loop {
                    let line_len = reader.read_until(b'\n', &mut line).expect("could not read bytes");
                    if line_len == 0 {
                        break;
                    }
                    let (station, temp) = parse_line(&line);
                    let station = unsafe { String::from_utf8_unchecked(Vec::from(station)) };
                    let temp = parse_temp(temp);
                    let measurements_option = t_stations.get_mut(&station);
                    if let Some(measurements) = measurements_option {
                        measurements.update(temp);
                    } else {
                        let measurements = onebrc::StationMeasurements {
                            min: temp,
                            max: temp,
                            count: 1,
                            sum: temp,
                        };
                        t_stations.insert(station, measurements);
                    }
                    currposition += line.len() as u64;
                    if currposition >= end {
                        break;
                    }
                    line.clear();
                }
                let _ = tx.send(t_stations);
            });
        }
        drop(tx);
        while let Ok(t_stations) = rx.recv() {
            for (station, measurements) in t_stations.iter() {
                let joined_measurements_options = stations.get_mut(station.as_str());
                if let Some(joined_measurements) = joined_measurements_options {
                    joined_measurements.merge(measurements);
                } else {
                    stations.insert(station.to_owned(), *measurements);
                }
            }
        }
        let mut stations: Vec<String> = stations.iter().map(|(station, measurements)| {
            let measurements = measurements.to_string();
            #[cfg(feature = "json")]
            {
                format!("{{\"{station}\":\"{measurements}\"}}")
            }
            #[cfg(not(feature = "json"))]
            {
                format!("{station}={measurements}")
            }
        }).collect();
        stations.sort();
        let stations = stations.join(",");
        #[cfg(feature = "json")]
        {
            println!("\n\n[{stations}]");
        }
        #[cfg(not(feature = "json"))]
        {
            println!("\n\n{{{stations}}}");
        }
        println!("\n\nTime={} ms", now.elapsed().as_millis());
    });
}
