use polars::prelude::*;
use std::time::Instant;
use std::vec;

fn run_polars() -> Result<DataFrame, PolarsError> {
    let now = Instant::now();

    let f1: Field = Field::new("station", DataType::String);
    let f2: Field = Field::new("measure", DataType::Float64);
    let sc: Schema = Schema::from_iter(vec![f1, f2]);

    let q = LazyCsvReader::new("../../../measurements.txt")
        .has_header(false)
        .with_schema(Some(Arc::new(sc)))
        .with_separator(b';')
        .finish()?
        .group_by(vec![col("station")])
        .agg(vec![
            col("measure").alias("min").min(),
            col("measure").alias("mean").mean(),
            col("measure").alias("max").max(),
        ])
        .sort("station", Default::default())
        .with_streaming(true);

    let df = q.collect()?;

    println!("Time={} μs", now.elapsed().as_micros());

    Ok(df)
}

fn main() {
    run_polars();
}
