use std::collections::HashMap;
use std::fs::File;
use std::io::{BufRead, BufReader};
use std::time::Instant;

use onebrc::parse_line;

const DEFAULT_HASHMAP_LENGTH: usize = 10000;

fn main() {
    let now = Instant::now();
    let mut stations: HashMap<String, onebrc::StationMeasurements> =
        HashMap::with_capacity(DEFAULT_HASHMAP_LENGTH);

    let file = File::open("../../../measurements.txt").expect("File measurements.txt not found");
    let mut reader = BufReader::new(&file);
    let mut line = Vec::with_capacity(108);
    loop {
        let line_len = reader.read_until(b'\n', &mut line).expect("could not read bytes");
        if line_len == 0 {
            break;
        }
        let (station, temp) = parse_line(&line);
        let station = unsafe { String::from_utf8_unchecked(Vec::from(station)) };
        let temp = onebrc::parse_temp(temp);
        let measurements_option = stations.get_mut(&station);
        if let Some(measurements) = measurements_option {
            measurements.update(temp);
        } else {
            let measurements = onebrc::StationMeasurements {
                min: temp,
                max: temp,
                count: 1,
                sum: temp,
            };
            stations.insert(station, measurements);
        }
        line.clear();
    }
    let mut stations: Vec<String> = stations.iter().map(|(station, measurements)| {
        let measurements = measurements.to_string();
        format!("{station}={measurements}")
    }).collect();
    stations.sort();
    let stations = stations.join(",");
    println!("{{{stations}}}");
    println!("Time={} μs", now.elapsed().as_micros());
}

